SystemOrganization addCategory: #'PN-SplitPasswordStore'!

Exception subclass: #SpsPasswordTooLong
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PN-SplitPasswordStore'!

Object subclass: #SpsDevRandom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PN-SplitPasswordStore'!

!SpsDevRandom commentStamp: '<historical>' prior: 0!
I am an interface for /dev/random on Unix-like platforms.!

!SpsDevRandom class methodsFor: 'accessing' stamp: 'PierceNg 8/11/2013 21:06'!
random160

	^ (FileSystem / 'dev' / 'random' ) readStreamDo: [ :s |
		s binary.
		(s next: 20) asByteArray ]! !

Object subclass: #SpsSplitPasswordStore
	instanceVariableNames: 'maxPasswordLength maxBlockLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PN-SplitPasswordStore'!

!SpsSplitPasswordStore commentStamp: '<historical>' prior: 0!
I am a secret-splitting password store. I split a given password into N pieces via bitwise XOR with N-1 random strings in turn, and serialize the N pieces into a disk file.!

!SpsSplitPasswordStore class methodsFor: 'retrieving' stamp: 'PierceNg 8/12/2013 22:56'!
readFrom: aFilename
	^ self new readFrom: aFilename asString: true! !

!SpsSplitPasswordStore class methodsFor: 'retrieving' stamp: 'PierceNg 8/11/2013 23:13'!
readFrom: aFilename asString: aBoolean
	^ self new readFrom: aFilename asString: aBoolean! !

!SpsSplitPasswordStore class methodsFor: 'storing' stamp: 'PierceNg 8/11/2013 23:11'!
split: aByteArray into: numberOfPieces andWriteTo: aFilename
	^ self new split: aByteArray into: numberOfPieces andWriteTo: aFilename! !

!SpsSplitPasswordStore methodsFor: 'initializing' stamp: 'PierceNg 8/11/2013 22:29'!
initialize

	maxPasswordLength := 16.
	maxBlockLength := 20.

	! !

!SpsSplitPasswordStore methodsFor: 'processing' stamp: 'PierceNg 8/11/2013 22:26'!
join: aColl
	| data |
	
	data := aColl first.
	aColl allButFirstDo: [ :x |
		data := data bitXor: x ].
	^ self unpad: data 
	
	! !

!SpsSplitPasswordStore methodsFor: 'processing' stamp: 'PierceNg 8/11/2013 21:51'!
pad: aByteArray
	^ self pad: aByteArray to: maxBlockLength
! !

!SpsSplitPasswordStore methodsFor: 'processing' stamp: 'PierceNg 8/11/2013 22:44'!
pad: aByteArray to: aLength
	| padSize pad |
	
	padSize := aLength - aByteArray size - 1.
	pad := SpsDevRandom random160 first: padSize.
	^ pad, aByteArray, padSize asByteArray
! !

!SpsSplitPasswordStore methodsFor: 'processing' stamp: 'PierceNg 8/12/2013 22:53'!
readFrom: aFilename
	^ self readFrom: aFilename asString: true! !

!SpsSplitPasswordStore methodsFor: 'processing' stamp: 'PierceNg 8/11/2013 22:57'!
readFrom: aFilename asString: aBoolean
	| data |
	
	data := self join: (FLMaterializer materializeFromFileNamed: aFilename).
	aBoolean ifTrue: [ ^ data asString ] ifFalse: [ ^ data ]! !

!SpsSplitPasswordStore methodsFor: 'processing' stamp: 'PierceNg 8/11/2013 22:44'!
split: aByteArray into: anInteger 
	| keys data |
	
	(aByteArray size > maxPasswordLength) ifTrue: [ SpsPasswordTooLong signal ].
	
	data := self pad: aByteArray.
	
	keys := OrderedCollection new: anInteger.
	(anInteger - 1) timesRepeat: [ 
		| k |
		k := SpsDevRandom random160.
		data := data bitXor: k.				
		keys add: k ].
	keys add: data.
	^ keys
	! !

!SpsSplitPasswordStore methodsFor: 'processing' stamp: 'PierceNg 8/11/2013 22:55'!
split: aByteArray into: anInteger andWriteTo: aFilename
	FLSerializer serialize: (self split: aByteArray into: anInteger) toFileNamed: aFilename
		! !

!SpsSplitPasswordStore methodsFor: 'processing' stamp: 'PierceNg 8/11/2013 22:27'!
unpad: aByteArray
	| padSize |
	
	padSize := aByteArray last asInteger.
	^ aByteArray copyFrom: padSize + 1 to: aByteArray size - 1
! !
